@@ -15,15 +15,15 @@
 
 NMSetting *_nm_connection_find_base_type_setting(NMConnection *connection);
 
-const char *_nm_connection_detect_slave_type(NMConnection *connection, NMSetting **out_s_port);
+const char *_nm_connection_detect_port_type(NMConnection *connection, NMSetting **out_s_port);
 
-gboolean _nm_connection_detect_slave_type_full(NMSettingConnection *s_con,
-                                               NMConnection        *connection,
-                                               const char         **out_slave_type,
-                                               const char         **out_normerr_slave_setting_type,
-                                               const char         **out_normerr_missing_slave_type,
-                                               const char **out_normerr_missing_slave_type_port,
-                                               GError     **error);
+gboolean _nm_connection_detect_port_type_full(NMSettingConnection *s_con,
+                                              NMConnection        *connection,
+                                              const char         **out_port_type,
+                                              const char         **out_normerr_port_setting_type,
+                                              const char         **out_normerr_missing_port_type,
+                                              const char **out_normerr_missing_port_type_port,
+                                              GError     **error);
 
 const char *_nm_connection_detect_bluetooth_type(NMConnection *self);
 
